@@ -4,7 +4,7 @@ use warnings;
 
 package Mxpress::PDF {
 	BEGIN {
-		our $VERSION = '0.18';
+		our $VERSION = '0.19';
 		our $AUTHORITY = 'cpan:LNATION';
 	};
 	use Zydeco;
@@ -607,6 +607,23 @@ package Mxpress::PDF {
 							push @line, shift(@paragraph);
 						}
 
+						if (!@line && @paragraph) {
+							my $temp = shift(@paragraph);
+							my @letters = split //, $temp;
+							my ($wid, $lin) = 0;
+							while ($wid < $lw) {
+								my $letter = shift @letters;
+								$wid += $text->advancewidth($letter);
+								$lin .= $letter;
+							}
+							$width{$lin} = $wid;
+							my $rest = join '', @letters;
+							$width{$rest} = $text->advancewidth($rest);
+							$line_width += $wid;
+							push @line, $lin;
+							unshift(@paragraph, $rest);
+						}
+
 						my ($wordspace, $align);
 						if ($self->align eq 'fulljustify' or $self->align eq 'justify' and @paragraph) {
 							if (scalar(@line) == 1) {
@@ -716,6 +733,48 @@ package Mxpress::PDF {
 					$class->generic_new($file, %args);
 				}
 			}
+			class +H1 {
+				factory h1 (Object $file, Map %args) {
+					$args{font}->{size} ||= 50/pt;
+					$args{font}->{line_height} ||= 40/pt;
+					$class->generic_new($file, %args);
+				}
+			}
+			class +H2 {
+				factory h2 (Object $file, Map %args) {
+					$args{font}->{size} ||= 40/pt;
+					$args{font}->{line_height} ||= 30/pt;
+					$class->generic_new($file, %args);
+				}
+			}
+			class +H3 {
+				factory h3 (Object $file, Map %args) {
+					$args{font}->{size} ||= 30/pt;
+					$args{font}->{line_height} ||= 20/pt;
+					$class->generic_new($file, %args);
+				}
+			}
+			class +H4 {
+				factory h4 (Object $file, Map %args) {
+					$args{font}->{size} ||= 25/pt;
+					$args{font}->{line_height} ||= 15/pt;
+					$class->generic_new($file, %args);
+				}
+			}
+			class +H5 {
+				factory h5 (Object $file, Map %args) {
+					$args{font}->{size} ||= 20/pt;
+					$args{font}->{line_height} ||= 15/pt;
+					$class->generic_new($file, %args);
+				}
+			}
+			class +H6 {
+				factory h6 (Object $file, Map %args) {
+					$args{font}->{size} ||= 15/pt;
+					$args{font}->{line_height} ||= 10/pt;
+					$class->generic_new($file, %args);
+				}
+			}
 		}
 		class +TOC::Outline extends Plugin::Text {
 			has outline (type => Object);
@@ -789,7 +848,7 @@ package Mxpress::PDF {
 					count => 0,
 					toc_line_offset => $args{toc_line_offset} || 0,
 					padding => $args{padding} || 0,
-					levels => [qw/title subtitle subsubtitle/],
+					levels => $args{levels} || [qw/title subtitle subsubtitle/],
 					indent => $args{indent} || 5,
 					($args{font} ? (font => $args{font}) : ())
 				);
@@ -1156,7 +1215,7 @@ Mxpress::PDF - PDF
 
 =head1 VERSION
 
-Version 0.18
+Version 0.19
 
 =cut
 
